@@ -22,7 +22,7 @@
 #include <locale.h>
 #include <glibmm.h>
 
-#include "initializer.hh"
+#include "initializer.h"
 
 namespace Glib {
 namespace Util {
@@ -30,7 +30,7 @@ namespace Util {
 Initializer::Initializer()
 {
     setlocale(LC_ALL, "");
-    Glib::thread_init();
+    if (! Glib::thread_supported()) Glib::thread_init();
 }
 
 Initializer::~Initializer()
