@@ -87,9 +87,11 @@ public:
     void on_tile_focus_in(Tile& tile);
     void on_tile_activated(Tile& tile);
 
-    // TilePageNavigator signal handlers
-    void on_show_next_page();
+    // PageNavigator signal handlers
+    void on_show_first_page();
     void on_show_previous_page();
+    void on_show_next_page();
+    void on_show_last_page();
 
     // Child widgets
     auto_ptr<PageNavigator> navigator_;
@@ -149,11 +151,17 @@ TileView::Private::Private(bool use_page_view)
 void
 TileView::Private::connect_signals()
 {
-    navigator_->signal_next().connect(
-        sigc::mem_fun(*this, &TileView::Private::on_show_next_page));
-
+    navigator_->signal_first().connect(
+        sigc::mem_fun(*this, &TileView::Private::on_show_first_page));
+    
     navigator_->signal_previous().connect(
         sigc::mem_fun(*this, &TileView::Private::on_show_previous_page));
+    
+    navigator_->signal_next().connect(
+        sigc::mem_fun(*this, &TileView::Private::on_show_next_page));
+    
+    navigator_->signal_last().connect(
+        sigc::mem_fun(*this, &TileView::Private::on_show_last_page));
 }
 
 // Registers the tile data in our internal data structure, determines
@@ -388,6 +396,26 @@ TileView::Private::update_navigator_page_info_label()
                             static_cast<int>(tiles_.size())));
 }
 
+void
+TileView::Private::on_show_first_page()
+{
+    if (! paginate_) return;
+
+    current_page_ = 1;
+    reload_container();
+    update_navigator_page_info_label();
+}
+
+void
+TileView::Private::on_show_previous_page()
+{
+    if ((! paginate_) || (current_page_ == 1)) return;
+
+    --current_page_;
+    reload_container();
+    update_navigator_page_info_label();
+}
+
 void
 TileView::Private::on_show_next_page()
 {
@@ -401,11 +429,11 @@ TileView::Private::on_show_next_page()
 }
 
 void
-TileView::Private::on_show_previous_page()
+TileView::Private::on_show_last_page()
 {
-    if ((! paginate_) || (current_page_ == 1)) return;
+    if (! paginate_) return;
 
-    --current_page_;
+    current_page_ = get_page_count();
     reload_container();
     update_navigator_page_info_label();
 }
@@ -515,7 +543,7 @@ TileView::add_tile(Tile* tile)
 Tile*
 TileView::get_selection()
 {
-    return priv_->selected_tile_->tile;
+    return (priv_->selected_tile_) ? priv_->selected_tile_->tile : 0;
 }
 
 TileView::SignalTileActivated&
@@ -548,6 +576,12 @@ TileView::set_navigator_title_markup(const Glib::ustring& marked_up_title)
     priv_->navigator_->set_title_markup(marked_up_title);
 }
 
+PageNavigator&
+TileView::get_page_navigator()
+{
+    return *(priv_->navigator_);
+}
+
 void
 TileView::set_page_view(bool use_page_view)
 {
